/* Testing imports */
import { mount } from "@vue/test-utils";
import { createTestingPinia } from "@pinia/testing";

/* Vue i18n */
import i18n, { def } from "@/plugins/vue-i18n";
import { ProjectI18nMessages } from "@/modules/project/i18n/index";
i18n.global.availableLocales.forEach((locale) => {
  i18n.global.setLocaleMessage(locale, def[locale]); // default locale messages
  i18n.global.mergeLocaleMessage(locale, ProjectI18nMessages[locale]); // append the locale messages for the component
});

/* Additional Dependencies */

/* Tested Component */
import CreateProject from "./CreateProject.vue";

/* Import of relevant mockup data */
import { testProjectForCreation } from "@/data/mockup/testProject";

describe("CreateProject.vue", () => {
  /* Describe Pre-initialization steps */

  /* Description of the test */
  test("ParentId from project is not saved to local storage.", async () => {
    // Test Pre-initialization steps

    // Mount the Component
    const wrapper = mount(CreateProject, {
      pinia: createTestingPinia({
        createSpy: vitest.fn,
      }),
    });
    await wrapper.vm.$nextTick();

    wrapper.vm.projectForCreation = Object.assign({}, testProjectForCreation);

    // Check initial state of parentId
    expect(wrapper.vm.projectForCreation.parentId).toBeDefined();

    await wrapper.vm.$nextTick();
    // save to local storage
    await wrapper.vm.saveProjectForCreationLocalStorage();

    await wrapper.vm.$nextTick();

    // read local storage

    const projectJson = localStorage.getItem(wrapper.vm.projectLocalStorageKey);
    let project = null;

    if (projectJson != null) {
      project = JSON.parse(projectJson);
    }
    // assert
    expect(projectJson).not.toBeNull();
    expect(project).not.toBeNull();
    expect(project.projectId).toBeUndefined();

    wrapper.unmount();
  });

  test("ParentId from project is correct after load from local storage.", async () => {
    /* Test Pre-initialization steps */

    /* Mount the Component */
    const wrapper = mount(CreateProject, {
      pinia: createTestingPinia({
        createSpy: vitest.fn,
      }),
    });
    await wrapper.vm.$nextTick();

    wrapper.vm.projectForCreation = Object.assign({}, testProjectForCreation);
    await wrapper.vm.$nextTick();
    // Check initial state of parentId
    expect(wrapper.vm.projectForCreation.parentId).toBeDefined();

    await wrapper.vm.$nextTick();
    // save to local storage
    await wrapper.vm.saveProjectForCreationLocalStorage();

    await wrapper.vm.$nextTick();

    // change parentId after save to the local storage
    const testParentId = "123-abc-def-456";
    wrapper.vm.projectForCreation.parentId = testParentId;

    await wrapper.vm.$nextTick();
    // load from local storage
    await wrapper.vm.restoreProjectForCreationLocalStorage();
    await wrapper.vm.$nextTick();

    // check if set parentId is still the one we set
    expect(wrapper.vm.projectForCreation.parentId).toBe(testParentId);
    wrapper.unmount();
  });
});
